package com.yizhi.core.application.file.util;

import com.aliyun.oss.ClientConfiguration;
import com.aliyun.oss.OSSClient;
import com.aliyun.oss.OSSException;
import com.aliyun.oss.model.CompleteMultipartUploadResult;
import com.aliyun.oss.model.PutObjectResult;
import com.aliyun.oss.model.UploadFileRequest;
import com.aliyun.oss.model.UploadFileResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayInputStream;
import java.util.Calendar;

/**
 * @author wenjunlong
 */
public class OssUpload {

    private static String endpoint = "https://oss-cn-shanghai.aliyuncs.com";
    private static String accessKeyId = "LTAIeBliOjnD6v25";
    private static String accessKeySecret = "qd6iAV7tUvumvEiXxv9IGJbSdHcnmH";
    private static String bucketName = "cloud-wmy";

    private static String proxyUsername = null;    //代理用户
    private static String proxyPassword = null;    //代理密码

    private static final Logger LOGGER = LoggerFactory.getLogger(OssUpload.class);

    /**
     * 上传文件到阿里云
     *
     * @param uploadFile 上传文件完整路径(/test.xls)
     * @param key        上传到阿里云的文件名带后缀名(testaliyun.xls)
     * @return
     */
    public static String upload(String uploadFile, String key) {
        return upload(uploadFile, key, null, null);
    }

    /**
     * 上传文件到阿里云，支持代理
     *
     * @param uploadFile 上传文件完整路径(/test.xls)
     * @param key        上传到阿里云的文件名带后缀名(testaliyun.xls)
     * @return
     */
    public static String upload(String uploadFile, String key, String proxyUrl, Integer proxyPort) {
        OSSClient ossClient = null;
        if (!StringUtils.isBlank(proxyUrl) && proxyPort != null && proxyPort != 0) {
            LOGGER.info("创建OSSClient代理连接：IP={}, PORT={}", proxyUrl, proxyPort);
            ClientConfiguration config = new ClientConfiguration();
            config.setProxyHost(proxyUrl);
            config.setProxyPort(proxyPort);
            config.setProxyPassword(proxyPassword);
            config.setProxyUsername(proxyUsername);

            ossClient = new OSSClient(endpoint, accessKeyId, accessKeySecret, config);
        } else {
            LOGGER.info("创建OSSClient正常连接");
            ossClient = new OSSClient(endpoint, accessKeyId, accessKeySecret);
        }

        try {
            UploadFileRequest uploadFileRequest = new UploadFileRequest(bucketName, key);
            // 待上传的本地文件
            uploadFileRequest.setUploadFile(uploadFile);
            // 设置并发下载数，默认1
            uploadFileRequest.setTaskNum(5);
            // 设置分片大小，默认100KB
            uploadFileRequest.setPartSize(1024 * 1024 * 1);
            // 开启断点续传，默认关闭
            uploadFileRequest.setEnableCheckpoint(true);

            UploadFileResult uploadResult = ossClient.uploadFile(uploadFileRequest);

            CompleteMultipartUploadResult multipartUploadResult =
                    uploadResult.getMultipartUploadResult();

            System.out.println(multipartUploadResult.getLocation());

            return multipartUploadResult.getLocation();
        } catch (OSSException oe) {
            System.out.println("Caught an OSSException, which means your request made it to OSS, "
                    + "but was rejected with an error response for some reason.");
            System.out.println("Error Message: " + oe.getErrorCode());
            System.out.println("Error Code:       " + oe.getErrorCode());
            System.out.println("Request ID:      " + oe.getRequestId());
            System.out.println("Host ID:           " + oe.getHostId());
            return "error";
        } catch (Throwable e) {
            e.printStackTrace();
            return "error";
        } finally {
            ossClient.shutdown();
        }
    }

    /**
     * 上传文件到oss，上传成功返回url地址，复制返回空字符串
     *
     * @param data
     * @param filename
     * @return url有效期为2年
     */
    public static String upload(byte[] data, String filename) {
        String ret = "";
        OSSClient ossClient = null;
        try {
            ossClient = new OSSClient(endpoint, accessKeyId, accessKeySecret);
            PutObjectResult result = ossClient.putObject(bucketName, filename, new ByteArrayInputStream(data));
            ret = result.getRequestId();
        } catch (OSSException e) {
            ret = e.getMessage();
        } catch (com.aliyun.oss.ClientException e) {
            ret = e.getMessage();
        }
        // 关闭OSSClient。
        ossClient.shutdown();
        // 设置URL过期时间为1小时。
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.YEAR, 2);//增加一年
        //cal.add(Calendar.DATE, n);//增加一天
        //cal.add(Calendar.DATE, -10);//减10天
        //cal.add(Calendar.MONTH, 1);//增加一个月
        //Date expiration = new Date(System.currentTimeMillis() + (1000 * 24*3600 * 1000));
        ret = ossClient.generatePresignedUrl(bucketName, filename, cal.getTime()).toString();
        return ret;
    }

    public static void main(String[] args) {
        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                System.out.println(OssUpload.upload("D:\\importpath\\org_import_template.xlsx", "org_import_template.xlsx"));
            }
        });
        thread.start();
    }
}
