package com.yizhi.application.filter;

import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.charset.Charset;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import javax.servlet.http.HttpServletRequest;

import com.yizhi.core.application.cache.CacheNamespace;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StreamUtils;

import com.netflix.zuul.ZuulFilter;
import com.netflix.zuul.context.RequestContext;
import org.springframework.util.StringUtils;

/**
 * @ClassName LogPostFilter
 * @Description 请求日志 先记录请求到达时间点，存放redis --> 后面交给 LogPostFilter 处理
 * @Author shengchenglong
 * @DATE 2021/1/11 11:19
 * @Version 1.0
 */
@Component
@Slf4j
public class LogPreFilter extends BaseZuulFilter {

	public static final String GATEWAY_LOG_START_TIME_HEADER = "gateway_log_start_time";

	@Value("${gateway.logSwitch:false}")
	private Boolean logSwitch;

	@Autowired
	private RedisTemplate redisTemplate;

	@Override
	public boolean shouldFilter() {
		return logSwitch;
	}

	@Override
	public Object run() {

		RequestContext.getCurrentContext().addZuulRequestHeader(GATEWAY_LOG_START_TIME_HEADER, String.valueOf(System.currentTimeMillis()));

		return null;
	}
}

